@@ -33,12 +33,29 @@
 #include "nmv-i-perspective.h"
 #include "nmv-i-conf-mgr.h"
 #include <libgnomevfs/gnome-vfs-init.h>
+#include "nmv-log-stream-utils.h"
 
 using namespace std ;
 using namespace nemiver ;
 using namespace nemiver::common ;
 namespace nemiver {
 
+static const UString CONF_KEY_NEMIVER_WINDOW_WIDTH =
+                "/apps/nemiver/workbench/window-width" ;
+static const UString CONF_KEY_NEMIVER_WINDOW_HEIGHT =
+                "/apps/nemiver/workbench/window-height" ;
+static const UString CONF_KEY_NEMIVER_WINDOW_POSITION_X =
+                "/apps/nemiver/workbench/window-position-x" ;
+static const UString CONF_KEY_NEMIVER_WINDOW_POSITION_Y =
+                "/apps/nemiver/workbench/window-position-y" ;
+static const UString CONF_KEY_NEMIVER_WINDOW_MAXIMIZED =
+                "/apps/nemiver/workbench/window-maximized" ;
+static const UString CONF_KEY_NEMIVER_WINDOW_MINIMUM_WIDTH=
+                "/apps/nemiver/workbench/window-minimum-width" ;
+static const UString CONF_KEY_NEMIVER_WINDOW_MINIMUM_HEIGHT=
+                "/apps/nemiver/workbench/window-minimum-height" ;
+
+
 class WorkbenchStaticInit {
     WorkbenchStaticInit ()
     {
@@ -73,11 +90,13 @@ private:
     //************************
     void on_quit_menu_item_action () ;
     void on_about_menu_item_action () ;
+    void on_shutting_down_signal () ;
     //************************
     //</slots (signal callbacks)>
     //************************
 
     void init_glade () ;
+    void init_window () ;
     void init_actions () ;
     void init_menubar () ;
     void init_toolbar () ;
@@ -86,13 +105,19 @@ private:
                                    list<Gtk::Widget*> &a_tbs) ;
     void add_perspective_body (IPerspectiveSafePtr &a_perspective,
                                Gtk::Widget *a_body) ;
+    bool remove_perspective_body (IPerspectiveSafePtr &a_perspective) ;
+    void remove_all_perspective_bodies () ;
     void select_perspective (IPerspectiveSafePtr &a_perspective) ;
 
+    void save_window_geometry () ;
+
+
     void do_init ()
     {
         WorkbenchStaticInit::do_init ();
     }
     void shut_down () ;
+    bool on_delete_event (GdkEventAny* event) ;
 
 public:
     Workbench () ;
@@ -131,11 +156,14 @@ struct Workbench::Priv {
 
     Priv () :
         initialized (false),
-        main (NULL),
-        root_window (NULL),
-        menubar (NULL),
-        toolbar_container (NULL)
-    {}
+        main (0),
+        root_window (0),
+        menubar (0),
+        toolbar_container (0),
+        bodies_container (0)
+    {
+    }
+
 };//end Workbench::Priv
 
 #ifndef CHECK_WB_INIT
@@ -149,15 +177,43 @@ struct Workbench::Priv {
 //*********************
 //signal slots methods
 //*********************
+bool
+Workbench::on_delete_event (GdkEventAny* a_event)
+{
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+    NEMIVER_TRY
+    // use event so that compilation doesn't fail with -Werror :(
+    if (a_event) {}
+
+    // clicking the window manager's X and shutting down the with Quit menu item
+    // should do the same thing
+    on_quit_menu_item_action () ;
+    NEMIVER_CATCH
+
+    //keep propagating
+    return false;
+}
+
 void
 Workbench::on_quit_menu_item_action ()
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+    NEMIVER_TRY
+
     shut_down () ;
+
+    NEMIVER_CATCH
 }
 
 void
 Workbench::on_about_menu_item_action ()
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+    NEMIVER_TRY
+
     Gtk::AboutDialog dialog;
     dialog.set_name (PACKAGE_NAME);
     dialog.set_version (PACKAGE_VERSION);
@@ -189,16 +245,29 @@ Workbench::on_about_menu_item_action ()
     dialog.set_license(license);
 
     dialog.run ();
+
+    NEMIVER_CATCH
+}
+
+void
+Workbench::on_shutting_down_signal ()
+{
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+    NEMIVER_TRY
+    save_window_geometry () ;
+    NEMIVER_CATCH
 }
 
 Workbench::Workbench ()
 {
-    m_priv = new Priv ();
+    m_priv.reset (new Priv ());
 }
 
 Workbench::~Workbench ()
 {
-    m_priv = NULL ;
+    remove_all_perspective_bodies () ;
+    LOG_D ("delete", "destructor-domain") ;
 }
 
 void
@@ -213,6 +282,7 @@ Workbench::get_info (Info &a_info) const
 void
 Workbench::do_init (Gtk::Main &a_main)
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
 
     DynamicModule::Loader *loader = get_module_loader () ;
     THROW_IF_FAIL (loader) ;
@@ -223,6 +293,7 @@ Workbench::do_init (Gtk::Main &a_main)
     m_priv->main = &a_main ;
 
     init_glade () ;
+    init_window () ;
     init_actions () ;
     init_menubar () ;
     init_toolbar () ;
@@ -230,7 +301,8 @@ Workbench::do_init (Gtk::Main &a_main)
 
     m_priv->initialized = true ;
 
-    m_priv->plugin_manager  = new PluginManager (*dynmod_manager) ;
+    m_priv->plugin_manager =
+        PluginManagerSafePtr (new PluginManager (*dynmod_manager)) ;
 
     NEMIVER_TRY
         m_priv->plugin_manager->load_plugins () ;
@@ -239,26 +311,36 @@ Workbench::do_init (Gtk::Main &a_main)
         IPerspectiveSafePtr perspective ;
         Plugin::EntryPointSafePtr entry_point ;
         list<Gtk::Widget*> toolbars ;
-        IWorkbenchSafePtr thiz (this, true) ;
 
         //**************************************************************
         //store the list of perspectives we may have loaded as plugins,
         //and init each of them.
         //**************************************************************
+        IWorkbench *workbench = dynamic_cast<IWorkbench*> (this) ;
         for (plugin_iter = m_priv->plugin_manager->plugins_map ().begin () ;
              plugin_iter != m_priv->plugin_manager->plugins_map ().end ();
              ++plugin_iter) {
+             LOG_D ("plugin '"
+                    << plugin_iter->second->descriptor ()->name ()
+                    << "' refcount: "
+                    << (int) plugin_iter->second->get_refcount (),
+                    "refcount-domain") ;
             if (plugin_iter->second && plugin_iter->second->entry_point_ptr ()) {
                 entry_point = plugin_iter->second->entry_point_ptr () ;
                 perspective = entry_point.do_dynamic_cast<IPerspective> () ;
                 if (perspective) {
                     m_priv->perspectives.push_front (perspective) ;
-                    perspective->do_init (thiz) ;
+                    perspective->do_init (workbench) ;
                     perspective->edit_workbench_menu () ;
                     toolbars.clear () ;
                     perspective->get_toolbars (toolbars) ;
                     add_perspective_toolbars (perspective, toolbars) ;
                     add_perspective_body (perspective, perspective->get_body ()) ;
+                    LOG_D ("perspective '"
+                           << perspective->get_perspective_identifier ()
+                           << "' refcount: "
+                           << (int) perspective->get_refcount (),
+                           "refcount-domain") ;
                 }
             }
         }
@@ -272,6 +354,8 @@ Workbench::do_init (Gtk::Main &a_main)
 void
 Workbench::shut_down ()
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
     shutting_down_signal ().emit () ;
     m_priv->main->quit () ;
 }
@@ -279,6 +363,8 @@ Workbench::shut_down ()
 Glib::RefPtr<Gtk::ActionGroup>
 Workbench::get_default_action_group ()
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
     CHECK_WB_INIT ;
     THROW_IF_FAIL (m_priv) ;
     return m_priv->default_action_group ;
@@ -373,19 +459,74 @@ Workbench::shutting_down_signal ()
 void
 Workbench::init_glade ()
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
     THROW_IF_FAIL (m_priv) ;
+
     UString file_path = env::build_path_to_glade_file ("workbench.glade") ;
     m_priv->glade = Gnome::Glade::Xml::create (file_path) ;
     THROW_IF_FAIL (m_priv->glade) ;
 
-    m_priv->root_window =
-       ui_utils::get_widget_from_glade<Gtk::Window> (m_priv->glade, "workbench");
+    m_priv->root_window.reset
+           (ui_utils::get_widget_from_glade<Gtk::Window> (m_priv->glade,
+                                                          "workbench"));
+   // m_priv->root_window->property_allow_shrink ().set_value (true) ;
     m_priv->root_window->hide () ;
 }
+void
+
+Workbench::init_window ()
+{
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+    THROW_IF_FAIL (m_priv) ;
+    THROW_IF_FAIL (m_priv->root_window) ;
+    IConfMgr &conf_mgr = get_configuration_manager () ;
+
+    int width=0, height=0, pos_x=0, pos_y=0 ;
+    LOG_DD ("getting windows geometry from confmgr ...") ;
+    conf_mgr.get_key_value (CONF_KEY_NEMIVER_WINDOW_WIDTH, width) ;
+    conf_mgr.get_key_value (CONF_KEY_NEMIVER_WINDOW_HEIGHT, height) ;
+    conf_mgr.get_key_value (CONF_KEY_NEMIVER_WINDOW_POSITION_X, pos_x) ;
+    conf_mgr.get_key_value (CONF_KEY_NEMIVER_WINDOW_POSITION_Y, pos_y) ;
+    bool maximized=false ;
+    conf_mgr.get_key_value (CONF_KEY_NEMIVER_WINDOW_MAXIMIZED, maximized) ;
+    LOG_DD ("got windows geometry from confmgr.") ;
+
+    if (width) {
+        LOG_DD ("restoring windows geometry from confmgr ...") ;
+        m_priv->root_window->resize (width, height) ;
+        m_priv->root_window->move (pos_x, pos_y) ;
+        if (maximized) {
+            m_priv->root_window->maximize () ;
+        }
+        LOG_DD ("restored windows geometry from confmgr") ;
+    } else {
+        LOG_DD ("null window geometry from confmgr.") ;
+    }
+
+    //set the minimum width/height of nemiver, just in case.
+    width=700, height=500 ;
+    conf_mgr.get_key_value (CONF_KEY_NEMIVER_WINDOW_MINIMUM_WIDTH, width) ;
+    conf_mgr.get_key_value (CONF_KEY_NEMIVER_WINDOW_MINIMUM_HEIGHT, height) ;
+    m_priv->root_window->set_size_request (width, height) ;
+    LOG_DD ("set windows min size to ("
+            << (int) width
+            << ","
+            << (int) height
+            << ")") ;
+
+    shutting_down_signal ().connect (sigc::mem_fun
+                        (*this, &Workbench::on_shutting_down_signal)) ;
+    m_priv->root_window->signal_delete_event ().connect (
+            sigc::mem_fun (*this, &Workbench::on_delete_event));
+}
 
 void
 Workbench::init_actions ()
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
     Gtk::StockID nil_stock_id ("") ;
     sigc::slot<void> nil_slot ;
     using ui_utils::ActionEntry ;
@@ -455,6 +596,8 @@ Workbench::init_actions ()
 void
 Workbench::init_menubar ()
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
     THROW_IF_FAIL (m_priv && m_priv->default_action_group) ;
 
 
@@ -473,6 +616,8 @@ Workbench::init_menubar ()
 void
 Workbench::init_toolbar ()
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
     m_priv->toolbar_container =
         ui_utils::get_widget_from_glade<Gtk::Notebook> (m_priv->glade,
                                                         "toolbarcontainer") ;
@@ -481,6 +626,7 @@ Workbench::init_toolbar ()
 void
 Workbench::init_body ()
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
     m_priv->bodies_container =
         ui_utils::get_widget_from_glade<Gtk::Notebook> (m_priv->glade,
                                                         "bodynotebook") ;
@@ -490,6 +636,8 @@ void
 Workbench::add_perspective_toolbars (IPerspectiveSafePtr &a_perspective,
                                      list<Gtk::Widget*> &a_tbs)
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
     if (a_tbs.empty ()) {return ;}
 
     SafePtr<Gtk::Box> box (Gtk::manage (new Gtk::VBox)) ;
@@ -510,15 +658,81 @@ void
 Workbench::add_perspective_body (IPerspectiveSafePtr &a_perspective,
                                  Gtk::Widget *a_body)
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
     if (!a_body || !a_perspective) {return;}
 
     m_priv->bodies_index_map[a_perspective.get ()] =
         m_priv->bodies_container->insert_page (*a_body, -1);
 }
 
+bool
+Workbench::remove_perspective_body (IPerspectiveSafePtr &a_perspective)
+{
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+    THROW_IF_FAIL (m_priv) ;
+    THROW_IF_FAIL (m_priv->bodies_container) ;
+
+    if (!a_perspective) {return false;}
+
+    map<IPerspective*, int>::iterator it;
+    it = m_priv->bodies_index_map.find (a_perspective.get ()) ;
+    if (it == m_priv->bodies_index_map.end ()) {
+        return false ;
+    }
+    m_priv->bodies_container->remove_page (it->second)  ;
+    m_priv->bodies_index_map.erase (it) ;
+    return true ;
+}
+
+void
+Workbench::save_window_geometry () 
+{
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+    THROW_IF_FAIL (m_priv) ;
+    THROW_IF_FAIL (m_priv->root_window) ;
+    IConfMgr &conf_mgr = get_configuration_manager () ;
+
+    int width=0, height=0, pos_x=0, pos_y=0 ;
+    m_priv->root_window->get_size (width, height) ;
+    m_priv->root_window->get_position (pos_x, pos_y) ;
+    bool maximized = (m_priv->root_window->get_window()->get_state()
+                      & Gdk::WINDOW_STATE_MAXIMIZED);
+
+    conf_mgr.set_key_value (CONF_KEY_NEMIVER_WINDOW_MAXIMIZED, maximized) ;
+    if (!maximized) {
+        LOG_DD ("storing windows geometry to confmgr...") ;
+        conf_mgr.set_key_value (CONF_KEY_NEMIVER_WINDOW_WIDTH, width) ;
+        conf_mgr.set_key_value (CONF_KEY_NEMIVER_WINDOW_HEIGHT, height) ;
+        conf_mgr.set_key_value (CONF_KEY_NEMIVER_WINDOW_POSITION_X, pos_x) ;
+        conf_mgr.set_key_value (CONF_KEY_NEMIVER_WINDOW_POSITION_Y, pos_y) ;
+        LOG_DD ("windows geometry stored to confmgr") ;
+    } else {
+        LOG_DD ("windows was maximized, didn't store its geometry") ;
+    }
+}
+
+void
+Workbench::remove_all_perspective_bodies ()
+{
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+    map<IPerspective*, int>::iterator it ;
+    for (it = m_priv->bodies_index_map.begin ();
+         it != m_priv->bodies_index_map.end ();
+         ++it) {
+        m_priv->bodies_container->remove_page (it->second) ;
+    }
+    m_priv->bodies_index_map.clear () ;
+}
+
 void
 Workbench::select_perspective (IPerspectiveSafePtr &a_perspective)
 {
+    LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
     THROW_IF_FAIL (m_priv) ;
     THROW_IF_FAIL (m_priv->toolbar_container) ;
     THROW_IF_FAIL (m_priv->bodies_container) ;
@@ -543,6 +757,7 @@ Workbench::select_perspective (IPerspectiveSafePtr &a_perspective)
     m_priv->bodies_container->set_current_page (body_index) ;
 }
 
+
 //the dynmod initial factory.
 extern "C" {
 bool
